-- Create base relations

CREATE TABLE "keylist" (
	"KeylistId" INTEGER NOT NULL,
	"UserId" INTEGER NOT NULL,
	PRIMARY KEY ("KeylistId", "UserId"));

CREATE TABLE "user" (
	"UserId" INTEGER NOT NULL PRIMARY KEY,
	"CreatedTime" DATETIME NOT NULL,
	"LastLoginTime" DATETIME,
	"DisplayName" VARCHAR(255),
	"AdditionalKeylistId" INTEGER,
	"LoginName" VARCHAR(255),
	"LoginPassword" VARCHAR(255));
CREATE UNIQUE INDEX "user_DisplayName" ON "user" ("DisplayName");
CREATE UNIQUE INDEX "user_LoginName" ON "user" ("LoginName");

CREATE TABLE "user_vcard" (
	"UserId" INTEGER NOT NULL,
	"Type" VARCHAR(255) NOT NULL,
	"Data" TEXT NOT NULL,
	"Visibility" TINYINT UNSIGNED NOT NULL DEFAULT 0,
	PRIMARY KEY ("UserId", "Type"));
CREATE INDEX "user_vcard_UserId" ON "user_vcard" ("UserId");

CREATE TABLE "user_config" (
	"UserId" INTEGER NOT NULL,
	"Name" VARCHAR(255) NOT NULL,
	"Data" VARCHAR(255) NOT NULL,
	PRIMARY KEY ("UserId", "Name"));
CREATE INDEX "user_config_UserId" ON "user_config" ("UserId");

CREATE TABLE "message" (
	"MessageId" INTEGER NOT NULL PRIMARY KEY,
	"CreatedTime" DATETIME NOT NULL,
	"Owner" INTEGER NOT NULL,
	"PrimaryReference" INTEGER,
	"EntryPoint" BOOLEAN NOT NULL DEFAULT FALSE,
	"PageName" VARCHAR(255),
	"ReadAccessKeylistId" INTEGER,
	"AlterAccessKeylistId" INTEGER,
	"ReplyAccessKeylistId" INTEGER,
	"ModerationFlags" TINYINT NOT NULL DEFAULT 0,
	"SearchOptimisedContent" MEDIUMTEXT,
	"SearchOptimisedSubject" VARCHAR(255),
	"SearchRevision" SMALLINT UNSIGNED,
	"LastBeginEdit" DATETIME);
CREATE UNIQUE INDEX "message_PageName" ON "message" ("PageName");

CREATE TABLE "message_reference" (
	"MessageId" INTEGER NOT NULL,
	"Reference" INTEGER NOT NULL,
	PRIMARY KEY ("MessageId", "Reference"));

CREATE TABLE "message_revision" (
	"MessageId" INTEGER NOT NULL,
	"RevisionNumber" SMALLINT UNSIGNED NOT NULL,
	"CreatedTime" DATETIME NOT NULL,
	"Author" INTEGER NOT NULL,
	"Subject" VARCHAR(255),
	"Content" MEDIUMTEXT,
	"HtmlContent" MEDIUMTEXT,
	"Summary" VARCHAR(255),
	"ModerationState" TINYINT UNSIGNED NOT NULL DEFAULT 0,
	"Data" BLOB,
	"HasData" BOOLEAN NOT NULL DEFAULT FALSE,
	PRIMARY KEY ("MessageId", "RevisionNumber"));
CREATE INDEX "message_revision_CreatedTime" ON "message_revision" ("CreatedTime");

CREATE TABLE "tag" (
	"TagId" INTEGER NOT NULL PRIMARY KEY,
	"TagName" VARCHAR(255) NOT NULL,
	"ParentTag" INTEGER,
	"Meaning" TINYINT UNSIGNED,
	"Description" VARCHAR(255),
	"UseAccessKeylistId" INTEGER,
	"ReadAccessKeylistId" INTEGER,
	"Flags" TINYINT UNSIGNED NOT NULL DEFAULT 0);

CREATE TABLE "message_revision_tag" (
	"MessageId" INTEGER NOT NULL,
	"RevisionNumber" SMALLINT UNSIGNED NOT NULL,
	"TagId" INTEGER NOT NULL,
	"TagData" VARCHAR(255),
	PRIMARY KEY ("MessageId", "RevisionNumber", "TagId"));

CREATE TABLE "message_rating" (
	"UserId" INTEGER NOT NULL,
	"MessageId" INTEGER NOT NULL,
	"Rating" TINYINT NOT NULL,
	"Weight" TINYINT NOT NULL,
	PRIMARY KEY ("UserId", "MessageId"));
CREATE INDEX "message_rating_MessageId" ON "message_rating" ("MessageId");

-- Create foreign keys for referential integrity
-- (SQLite does not enforce foreign key constraints directly, so we do that ourselves with the help of triggers.)
-- TODO: "ON DELETE CASCADE" triggers only handle the DELETE part, they won't enforce any data!
--       Need more triggers for INSERT and UPDATE (where required)

-- ALTER TABLE "keylist" ADD FOREIGN KEY ("UserId") REFERENCES "user" ("UserId") ON DELETE CASCADE;
CREATE TRIGGER "fkd_keylist_user_UserId"
BEFORE DELETE ON "user"
FOR EACH ROW BEGIN
	DELETE FROM "keylist" WHERE "UserId" = OLD."UserId";
END;

-- ALTER TABLE "user_vcard" ADD FOREIGN KEY ("UserId") REFERENCES "user" ("UserId") ON DELETE CASCADE;
CREATE TRIGGER "fkd_user_vcard_user_UserId"
BEFORE DELETE ON "user"
FOR EACH ROW BEGIN
	DELETE FROM "user_vcard" WHERE "UserId" = OLD."UserId";
END;

-- ALTER TABLE "user_config" ADD FOREIGN KEY ("UserId") REFERENCES "user" ("UserId") ON DELETE CASCADE;
CREATE TRIGGER "fkd_user_config_user_UserId"
BEFORE DELETE ON "user"
FOR EACH ROW BEGIN
	DELETE FROM "user_config" WHERE "UserId" = OLD."UserId";
END;

-- ALTER TABLE "message" ADD FOREIGN KEY ("Owner") REFERENCES "user" ("UserId");
CREATE TRIGGER "fkd_message_user_Owner"
BEFORE DELETE ON "user"
FOR EACH ROW BEGIN
	SELECT CASE
		WHEN ((SELECT "Owner" FROM "message" WHERE "Owner" = OLD."UserId") IS NOT NULL)
		THEN RAISE(ABORT, 'Delete on table "user" violates foreign key constraint "fkd_message_user_Owner"')
	END;
END;
-- ALTER TABLE "message" ADD FOREIGN KEY ("PrimaryReference") REFERENCES "message" ("MessageId") ON DELETE SET NULL;
CREATE TRIGGER "fkd_message_message_PrimaryReference"
BEFORE DELETE ON "message"
FOR EACH ROW BEGIN
	UPDATE "message" SET "PrimaryReference" = NULL WHERE "PrimaryReference" = OLD."MessageId";
END;
-- ALTER TABLE "message" ADD FOREIGN KEY ("MessageId", "SearchRevision") REFERENCES "message_revision" ("MessageId", "RevisionNumber");
-- TODO

-- ALTER TABLE "message_reference" ADD FOREIGN KEY ("MessageId") REFERENCES "message" ("MessageId") ON DELETE CASCADE;
CREATE TRIGGER "fkd_message_reference_message_MessageId"
BEFORE DELETE ON "message"
FOR EACH ROW BEGIN
	DELETE FROM "message_reference" WHERE "MessageId" = OLD."MessageId";
END;
-- ALTER TABLE "message_reference" ADD FOREIGN KEY ("Reference") REFERENCES "message" ("MessageId") ON DELETE CASCADE;
CREATE TRIGGER "fkd_message_reference_message_Reference"
BEFORE DELETE ON "message"
FOR EACH ROW BEGIN
	DELETE FROM "message_reference" WHERE "Reference" = OLD."MessageId";
END;

-- ALTER TABLE "message_revision" ADD FOREIGN KEY ("MessageId") REFERENCES "message" ("MessageId") ON DELETE CASCADE;
CREATE TRIGGER "fkd_message_revision_message_MessageId"
BEFORE DELETE ON "message"
FOR EACH ROW BEGIN
	DELETE FROM "message_revision" WHERE "MessageId" = OLD."MessageId";
END;
-- ALTER TABLE "message_revision" ADD FOREIGN KEY ("Author") REFERENCES "user" ("UserId");
CREATE TRIGGER "fkd_message_revision_user_Author"
BEFORE DELETE ON "user"
FOR EACH ROW BEGIN
	SELECT CASE
		WHEN ((SELECT "Author" FROM "message_revision" WHERE "Author" = OLD."UserId") IS NOT NULL)
		THEN RAISE(ABORT, 'Delete on table "user" violates foreign key constraint "fkd_message_revision_user_Author"')
	END;
END;

-- ALTER TABLE "tag" ADD FOREIGN KEY ("ParentTag") REFERENCES "tag" ("TagId");
CREATE TRIGGER "fkd_tag_tag_ParentTag"
BEFORE DELETE ON "tag"
FOR EACH ROW BEGIN
	SELECT CASE
		WHEN ((SELECT "ParentTag" FROM "tag" WHERE "ParentTag" = OLD."TagId") IS NOT NULL)
		THEN RAISE(ABORT, 'Delete on table "tag" violates foreign key constraint "fkd_tag_tag_ParentTag"')
	END;
END;

-- ALTER TABLE "message_revision_tag" ADD FOREIGN KEY ("MessageId", "RevisionNumber") REFERENCES "message_revision" ("MessageId", "RevisionNumber") ON DELETE CASCADE;
CREATE TRIGGER "fkd_message_revision_tag_message_rev_MsgIdRevNum"
BEFORE DELETE ON "message_revision"
FOR EACH ROW BEGIN
	DELETE FROM "message_revision_tag" WHERE "MessageId" = OLD."MessageId" AND "RevisionNumber" = OLD."RevisionNumber";
END;
-- ALTER TABLE "message_revision_tag" ADD FOREIGN KEY ("TagId") REFERENCES "tag" ("TagId") ON DELETE CASCADE;
CREATE TRIGGER "fkd_message_revision_tag_tag_TagId"
BEFORE DELETE ON "tag"
FOR EACH ROW BEGIN
	DELETE FROM "message_revision_tag" WHERE "TagId" = OLD."TagId";
END;

-- ALTER TABLE "message_rating" ADD FOREIGN KEY ("UserId") REFERENCES "user" ("UserId") ON DELETE CASCADE;
CREATE TRIGGER "fkd_message_rating_user_UserId"
BEFORE DELETE ON "user"
FOR EACH ROW BEGIN
	DELETE FROM "message_rating" WHERE "UserId" = OLD."UserId";
END;
-- ALTER TABLE "message_rating" ADD FOREIGN KEY ("MessageId") REFERENCES "message" ("MessageId") ON DELETE CASCADE;
CREATE TRIGGER "fkd_message_rating_message_MessageId"
BEFORE DELETE ON "message"
FOR EACH ROW BEGIN
	DELETE FROM "message_rating" WHERE "MessageId" = OLD."MessageId";
END;

