<?php
// UNB Components
// Copyright 2007-2010 by Yves Goergen
// Website: http://unclassified.de/go/unb2
// Licence: GNU General Public License (GPL) Version 3
//
// This file contains the UnbEnvironment class.

// Environment functions for compatibility with different server environments and to query
// information about the client (browser).
//
class UnbEnvironment
{
	private static $browserName = null;
	private static $browserVersion = null;
	private static $browserClass = null;
	private static $isBrowser = null;
	private static $searchTerms = null;

	private function __construct()
	{
	}

	// Sets up the environment for UNB.
	//
	// This method combines all of the specific methods that are available. This includes PHP
	// configuration like error reporting or multibyte string functions encoding and request
	// variables processing under various PHP configurations.
	//
	public static function SetupEnvironment()
	{
		self::SetupErrorReporting();
		self::SetupMultibyte();
		self::SetupMagicQuotesRuntime();
		self::SetupMagicQuotesGPC();
		self::SetupNull();
		self::SetupRegisterGlobals();
		self::EnableHttpCompression();
	}

	// Configures PHP error reporting.
	//
	public static function SetupErrorReporting()
	{
		// We don't want error messages to be displayed from now on
		error_reporting(E_ALL & ~E_NOTICE);
		#ini_set('display_errors', 0);
	}

	// Configures PHP multibyte string functions. This is strongly recommended to use to ensure UNB
	// works as expected.
	//
	public static function SetupMultibyte()
	{
		// All multibyte string functions will use UTF-8
		mb_internal_encoding('UTF-8');
	}

	// Configures PHP magic quoting at runtime.
	//
	public static function SetupMagicQuotesRuntime()
	{
		// Disable magic_quote_runtime, it's entirely useless
		set_magic_quotes_runtime(0);
	}

	// Configures PHP magic quoting for GET/POST/COOKIE variables.
	//
	// Since this setting cannot be simply switched off, its effects are actually undone instead.
	// Do not call this method more than once.
	//
	public static function SetupMagicQuotesGPC()
	{
		// Undo what magic_quotes_gpc (possibly) already did
		if (get_magic_quotes_gpc())
		{
			self::DeSlashArray($_GET);
			self::DeSlashArray($_POST);
			self::DeSlashArray($_REQUEST);
			self::DeSlashArray($_COOKIE);
			self::DeSlashArray($_FILES);
		}
	}

	// Filters all NUL bytes out of input parameters.
	//
	// Some PHP functions have shown vulnerabilities to strings with NUL bytes and since this kind
	// of data is not expected to be passed as external parameter anyway, all of those are filtered
	// out.
	//
	public static function SetupNull()
	{
		// Remove NUL characters from any user input for security reasons
		self::DeNulArray($_GET);
		self::DeNulArray($_POST);
		self::DeNulArray($_REQUEST);
		self::DeNulArray($_COOKIE);
		self::DeNulArray($_FILES);
	}

	// Configures PHP register globals setting.
	//
	// Since this setting cannot be simply switched off, its effects are actually undone instead.
	//
	public static function SetupRegisterGlobals()
	{
		// Unregister_globals :) for more security
		if (ini_get('register_globals'))
		{
			if (sizeof($_SESSION)) foreach (array_keys($_SESSION) as $key) unset($GLOBALS[$key]);
			if (sizeof($_GET)) foreach (array_keys($_GET) as $key) unset($GLOBALS[$key]);
			if (sizeof($_POST)) foreach (array_keys($_POST) as $key) unset($GLOBALS[$key]);
			if (sizeof($_COOKIE)) foreach (array_keys($_COOKIE) as $key) unset($GLOBALS[$key]);
			if (sizeof($_SERVER)) foreach (array_keys($_SERVER) as $key) unset($GLOBALS[$key]);
			if (is_array($_GET['GLOBALS'])) foreach (array_keys($_GET['GLOBALS']) as $key) unset($GLOBALS[$key]);
			if (is_array($_POST['GLOBALS'])) foreach (array_keys($_POST['GLOBALS']) as $key) unset($GLOBALS[$key]);
		}
	}

	// Enables HTTP output compression.
	//
	public static function EnableHttpCompression()
	{
		// Activate page compression if it's not active yet
		if (!ini_get('zlib.output_compression') && !ob_get_level())
			ob_start('ob_gzhandler');
		// Ensure that output buffering is turned on
		if (!ob_get_level())
			ob_start();
	}

	// Applies stripslashes() to each array key and value, recursing into sub-arrays.
	//
	// This is particularly useful when PHP's option magic_quote_gpc is enabled which is generally
	// a bad idea because it will mess up input from the user. The array passed to this function
	// is modified in-place, so there is no return value and little data gets copied around.
	//
	// in/out a = (array) Array to be processed
	//
	private static function DeSlashArray(&$a)
	{
		foreach ($a as $key => $value)
		{
			if (strpos($key, '\\') !== false)
			{
				unset($a[$key]);
				continue;
			}
			if (is_string($value))
			{
				$a[$key] = stripslashes($value);
			}
			elseif (is_array($value))
			{
				self::DeSlashArray($a[$key]);
			}
			#$key2 = stripslashes($key);
			#if ($key2 !== $key)
			#{
			#	unset($a[$key]);
			#}
			#$a[$key2] = $value;
		}
	}

	// Removes NUL characters from each array key and value, recursing into sub-arrays.
	//
	// This also removes \r characters as part of line breaks on the Windows platform.
	//
	// In the past, some of PHP's functions have shown vulnerabilities against NUL values in
	// strings passed to them. To generally eliminate any attack using this method, all of those
	// characters are removed from user input data. They are entirely useless for normal operation
	// anyway. The array passed to this function is modified in-place, so there is no return
	// value and little data gets copied around.
	//
	// in/out a = (array) Array to be processed
	//
	private static function DeNulArray(&$a)
	{
		foreach ($a as $key => $value)
		{
			if (strpos($key, "\x00") !== false)
			{
				unset($a[$key]);
				continue;
			}
			if (is_string($value))
			{
				$a[$key] = str_replace(array("\x00", "\r"), array('', ''), $value);
			}
			elseif (is_array($value))
			{
				self::DeNulArray($a[$key]);
			}
			#$key2 = str_replace("\x00", '', $key);
			#if ($key2 !== $key)
			#{
			#	unset($a[$key]);
			#}
			#$a[$key2] = $value;
		}
	}

	// Gets the referer URL for this request.
	//
	public static function GetReferer()
	{
		return isset($_SERVER['HTTP_REFERER']) ? $_SERVER['HTTP_REFERER'] : '';
	}

	// Gets a value indicating whether the referer URL is on the same domain as this page.
	//
	// Returns (bool)
	//
	public static function IsRefererMyDomain()
	{
		$referer = (isset($_SERVER['HTTP_REFERER']) ? $_SERVER['HTTP_REFERER'] : '');
		$ref_mydomain = false;
		if (preg_match('_^https?://(.*?)/_i', $referer, $m))
		{
			$referer = $m[1];   // find domain from referer
			$me = $_SERVER['SERVER_NAME'];   // find our own server name
			if (!strcasecmp(substr($me, 0, 4), 'www.')) $me = substr($me, 4);   // remove www. prefix
			if (!strcasecmp(substr($referer, 0, 4), 'www.')) $referer = substr($referer, 4);
			if (!strcasecmp($me, $referer)) $ref_mydomain = true;
		}
		return $ref_mydomain;
	}

	// Gets the client IP address
	//
	public static function GetRemoteAddress()
	{
		return $_SERVER['REMOTE_ADDR'];
	}

	// Gets the client's language preferences.
	//
	// Returns an array of all language codes, sorted by descending priority. The most preferred
	// language code is in the first item, the least preferred is in the last item. All language
	// codes are normalised to the form "aa" or "aa_AA".
	//
	public static function GetClientLanguage()
	{
		$languageCodes = array();
		$priorities = array();
		$acceptLanguage = $_SERVER['HTTP_ACCEPT_LANGUAGE'];
		while (preg_match('/^\s*([a-z]{2})(?:[-_]([a-z]{2}))?(?:;\s*q\s*=\s*(\d+(?:\.\d+)?))?(?:\s*,\s*(.*))?$/i', $acceptLanguage, $m))
		{
			$acceptLanguage = $m[4];
			$languageCodes[] = strtolower($m[1]) . ($m[2] ? '_' . strtoupper($m[2]) : '');
			$priorities[] = $m[3] ? intval(floatval($m[3]) * 100) : 100;
		}
		array_multisort($priorities, SORT_DESC, SORT_NUMERIC, $languageCodes);
		return $languageCodes;
	}

	// Detects the client browser and caches information about it.
	//
	// Exports self::$browserName, self::$browserVersion, self::$browserClass, self::$isBrowser
	//
	// TODO: add Google Chrome
	//
	private static function DetectBrowser()
	{
		$names = array(
			'Slurp' => 'search',
			'ZyBorg' => 'search',
			'Ask Jeeves' => 'search',
			'Opera' => 'opera',
			'iCab' => 'icab',
			'MSIE' => 'ie',
			'Firebird' => 'fx',
			'Firefox' => 'fx',
			'Minefield' => 'fx',
			'Thunderbird' => 'fx',
			'Galeon' => 'galeon',
			'Chrome' => 'chrome',
			'Safari' => 'safari',
			'Konqueror' => 'konq',
			'Camino' => 'cam',
			'Gecko' => 'moz',
			'Netscape' => 'nn',
			'Lynx' => 'text',
			'ELinks' => 'text',
			'Links' => 'text',
			'w3m' => 'text',
			'Googlebot' => 'search',
			'Mediapartners-Google' => 'search',
			'msnbot' => 'search',
			'ConveraMultiMediaCrawler' => 'search',
			'Spider' => 'search',
			'Scooter' => 'search',
			'WebCrawler' => 'search',
			'crawler' => 'search',
			'WiseCrawler' => 'search',
			'LinkWalker' => 'search',
			'Nutch' => 'search',
			'robot' => 'bot',
			'Indy Library' => 'bot',
			'Jetbot' => 'bot',
			'psbot' => 'bot',
			'ia_archiver' => 'bot',
			'Python-urllib' => 'bot',
			'Gigabot' => 'bot',
			'Firefly' => 'bot',
			'ichiro' => 'bot',
			'lwp-trivial' => 'bot',
			'LWP::Simple' => 'bot',
			'bot' => 'bot',
			'Wget' => 'download',
			'DA 4' => 'download',
			'NetAnts' => 'download',
			'Mass Downloader' => 'download',
			'GetRight' => 'download',
			'PHP' => 'bot',
			'Mozilla' => 'nn'
			);
		$raw_ua = $_SERVER['HTTP_USER_AGENT'];
		foreach ($names as $ua => $name)
		{
			if (stristr($raw_ua, $ua))
			{
				self::$browserName = $name;
				if ($ua == 'Gecko') $ua = 'rv:';
				elseif ($ua == 'Netscape') $ua .= '[0-9]?[ /]';
				else $ua .= '[ /]';
				if (preg_match('~' . $ua . '([0-9.]+)~', $raw_ua, $m)) self::$browserVersion = $m[1];
				break;
			}
		}

		// See if it's a bot or a real browser
		self::$isBrowser = true;
		if (self::$browserName == 'search' ||
			self::$browserName == 'bot' ||
			self::$browserName == 'download' ||
			self::$browserName == 'php')
		{
			self::$isBrowser = false;
		}

		// Group browsers in classes for easier matching
		$classes = array(
			'ie' => array('ie'),
			'opera' => array('opera'),
			'gecko' => array('moz', 'fx', 'cam'),
			'webkit' => array('chrome', 'konq', 'safari')
			);
		foreach ($classes as $c => $browsers)
		{
			if (in_array(self::$browserName, $browsers))
			{
				self::$browserClass = $c;
				break;
			}
		}
	}

	// Gets the client's browser name.
	//
	// The following browser names are supported:
	// * fx: Mozilla Firefox
	// * ie: Microsoft Internet Explorer
	// * opera: Opera
	// * safari: Apple Safari
	// * chrome: Google Chrome
	// * konq: Konqueror
	// * moz: Mozilla Browser
	// * nn: Netscape Navigator
	// * cam: Mozilla Camino
	// * icab: iCab
	// * galeon: Galeon
	// * text: Text-based browser
	// * search: Search engines
	// * bot: Some bot
	// * download: Some downloader
	//
	public static function GetBrowserName()
	{
		if (!isset(self::$browserName))
			self::DetectBrowser();
		return self::$browserName;
	}

	// Gets the client's browser version.
	//
	// Returns (string) version "number"
	//
	public static function GetBrowserVersion()
	{
		if (!isset(self::$browserVersion))
			self::DetectBrowser();
		return self::$browserVersion;
	}

	// Gets the client's browser class.
	//
	// The following browser names are supported:
	// * gecko: Mozilla-based browsers with Gecko rendering engine
	// * ie: Microsoft Internet Explorer
	// * opera: Opera from Norway
	// * webkit: Browsers with Webkit/KHTML rendering engine (Safari, Chrome, Konqueror)
	//
	public static function GetBrowserClass()
	{
		if (!isset(self::$browserClass))
			self::DetectBrowser();
		return self::$browserClass;
	}

	// Gets a value indicating whether the client is a real browser.
	//
	public static function IsBrowser()
	{
		if (!isset(self::$isBrowser))
			self::DetectBrowser();
		return self::$isBrowser;
	}

	// Gets the client's best supported XHTML MIME type.
	//
	public static function BestXHtml()
	{
		// Determine browser XHTML capabilities with HTTP Accept header
		$accept = explode(',', $_SERVER['HTTP_ACCEPT']);
		foreach ($accept as $h => $v) { $e = explode(';', $v); $accept[$h] = $e[0]; }

		if (in_array('application/xhtml+xml', $accept)) return 'application/xhtml+xml';
		elseif (in_array('text/xml', $accept)) return 'text/xml';
		elseif (in_array('text/html', $accept)) return 'text/html';
		return 'text/html';
	}

	// Gets a value indicating whether the best supported XHTML MIME type is XML-based.
	//
	// Returns (bool)
	//
	public static function UseXml()
	{
		return strstr(self::BestXHtml(), 'xml') != false;
	}

	// Gets the client operating system name and version
	//
	public static function GetClientSystemName()
	{
		$raw_ua = $_SERVER['HTTP_USER_AGENT'];
		if (stristr($raw_ua, 'Windows 95')) return 'win95';
		elseif (stristr($raw_ua, 'Win95')) return 'win95';
		elseif (stristr($raw_ua, 'Windows 98')) return 'win98';
		elseif (stristr($raw_ua, 'Win98')) return 'win95';
		elseif (stristr($raw_ua, 'Windows ME')) return 'winme';
		elseif (stristr($raw_ua, 'Windows NT 4')) return 'winnt';
		elseif (stristr($raw_ua, 'Windows NT 5.0')) return 'win2k';
		elseif (stristr($raw_ua, 'Windows NT 5.1')) return 'winxp';
		elseif (stristr($raw_ua, 'Windows NT 5.2')) return 'win2k3';
		elseif (stristr($raw_ua, 'Windows NT 6.0')) return 'winvista';
		elseif (stristr($raw_ua, 'Windows NT 6.1')) return 'win7';
		elseif (stristr($raw_ua, 'WinNT')) return 'winnt';
		elseif (stristr($raw_ua, 'Windows-NT')) return 'winnt';
		elseif (stristr($raw_ua, 'Windows NT')) return 'winnt';
		elseif (stristr($raw_ua, 'Win')) return 'win';
		elseif (stristr($raw_ua, 'Linux')) return 'linux';
		elseif (stristr($raw_ua, 'Unix')) return 'unix';
		elseif (stristr($raw_ua, 'Mac')) return 'mac';
		elseif (stristr($raw_ua, 'SunOS')) return 'sun';
		return '';
	}

	// Gets the client operating system class
	//
	public static function GetClientSystemClass()
	{
		$systemName = self::GetClientSystemName();
		if (strpos($systemName, 'win') !== false) return 'win';
		if (strpos($systemName, 'mac') !== false) return 'mac';
		if (strpos($systemName, 'linux') !== false) return 'unix';
		if (strpos($systemName, 'unix') !== false) return 'unix';
		if (strpos($systemName, 'sun') !== false) return 'unix';
		return '';
	}

	// Gets the search terms found in the referer, e.g. if clicked on a search engine link.
	//
	// Returns (array(string)) search terms array
	//
	public static function GetSearchTerms()
	{
		if (!isset(self::$searchTerms))
		{
			self::$searchTerms = array();
			$url = parse_url(self::GetReferer());
			if (preg_match('_google\.[a-z]{2,3}(?:\.[a-z]{2,3})?$_i', $url['host']) && $url['path'] == '/search')
			{
				// Referer is Google search
				parse_str($url['query'], $arr);
				$q = $arr['q'];
				if (get_magic_quotes_gpc()) $q = stripslashes($q);
				$q = str_replace("\x00", '', $q);
				$q = explode(' ', $q);
				foreach ($q as $a)
				{
					$a = trim($a);
					if ($a != '') $searchTerms[] = $a;
				}
			}
			// TODO: Add more search engines
		}
		return self::$searchTerms;
	}

	// Gets the local server operating system class
	//
	public static function GetServerSystemType()
	{
		if (substr(strtoupper(PHP_OS), 0, 3) == 'WIN') return 'win';
		if (substr(strtoupper(PHP_OS), 0, 5) == 'LINUX') return 'linux';
		return '';
	}

#	public static function GetPhpConfig()
#	{
#	}

} // class UnbEnvironment

?>